@@ -77,36 +77,27 @@ class AutobuildTool(autobuild_base.AutobuildBase):
                             help="build a specific build configuration\n(may be specified as comma separated values in $AUTOBUILD_CONFIGURATION)",
                             metavar='CONFIGURATION',
                             default=self.configurations_from_environment())
-        parser.add_argument('--use-cwd', dest='use_cwd', default=False, action="store_true",
-            help="build in current working directory")
 
     def run(self, args):
         config = configfile.ConfigurationDescription(args.config_file)
         current_directory = os.getcwd()
-        build_directory = config.make_build_directory(args.dry_run)
-        logger.debug("building in %s" % build_directory)
-        if not args.use_cwd:
-            os.chdir(build_directory)
         try:
             configure_first = not args.do_not_configure
-            if args.all:
-                build_configurations = config.get_all_build_configurations()
-            elif args.configurations:
-                build_configurations = \
-                    [config.get_build_configuration(name) for name in args.configurations]
-            else:
-                build_configurations = config.get_default_build_configurations()
+            build_configurations = common.select_configurations(args, config, "building for")
             if not build_configurations:
                 logger.warn("no applicable build configurations found, autobuild cowardly refuses to build nothing!")
                 logger.warn("did you remember to mark a build command as default? try passing 'default=true' to your 'autobuild edit build' command")
 
-            logger.debug("building for configuration(s) %r" % build_configurations)
             for build_configuration in build_configurations:
+                build_directory = config.make_build_directory(build_configuration, args.dry_run)
+                logger.debug("building in %s" % build_directory)
+                if not args.dry_run:
+                    os.chdir(build_directory)
                 if configure_first:
                     result = _configure_a_configuration(config, build_configuration,
                         args.build_extra_arguments, args.dry_run)
                     if result != 0:
-                        raise BuildError("configuring default configuration returned %d" % (result))                    
+                        raise BuildError("configuring default configuration returned %d" % (result))
                 result = _build_a_configuration(config, build_configuration,
                     args.build_extra_arguments, args.dry_run)
                 if result != 0:
@@ -136,7 +127,10 @@ def _build_a_configuration(config, build_configuration, extra_arguments, dry_run
         common_build_configuration = \
             config.get_build_configuration(build_configuration.name, 'common')
         parent_build = common_build_configuration.build
-    except:
+    except Exception, e:
+        if logger.getEffectiveLevel() <= logging.DEBUG:
+            logger.exception(e)
+        logger.debug('no common platform found')
         parent_build = None
     if build_configuration.build is not None:
         build_executable = copy.copy(build_configuration.build)
