@@ -62,26 +62,17 @@ class AutobuildTool(autobuild_base.AutobuildBase):
             help="build all configurations")
         parser.add_argument('additional_options', nargs="*", metavar='OPT',
             help="an option to pass to the configuration command" )
-        parser.add_argument('--use-cwd', dest='use_cwd', default=False, action="store_true",
-            help="configure in current working directory")
 
     def run(self, args):
         config = configfile.ConfigurationDescription(args.config_file)
         current_directory = os.getcwd()
-        build_directory = config.make_build_directory(args.dry_run)
-        logger.debug("configuring in %s" % build_directory)
-        if not ( args.use_cwd or args.dry_run ):
-            os.chdir(build_directory)
         try:
-            if args.all:
-                build_configurations = config.get_all_build_configurations()
-            elif args.configurations:
-                build_configurations = \
-                    [config.get_build_configuration(name) for name in args.configurations]
-            else:
-                build_configurations = config.get_default_build_configurations()
-            logger.debug("configuring for configuration(s) %r" % build_configurations)
+            build_configurations = common.select_configurations(args, config, "configuring for")
             for build_configuration in build_configurations:
+                build_directory = config.make_build_directory(build_configuration, args.dry_run)
+                logger.debug("configuring in %s" % build_directory)
+                if not args.dry_run:
+                    os.chdir(build_directory)
                 result = _configure_a_configuration(config, build_configuration,
                     args.additional_options, args.dry_run)
                 if result != 0:
