@@ -91,74 +91,6 @@ Supported platforms include: windows, darwin, linux, and a common platform
 to represent a platform-independent package.
 """
 
-def add_arguments(parser):
-    parser.description = "install artifacts of dependency packages for use during the build of the current package"
-    parser.add_argument(
-        'package',
-        nargs='*',
-        help='List of packages to consider for installation.')
-    parser.add_argument(
-        '--config-file',
-        default=configfile.AUTOBUILD_CONFIG_FILE,
-        dest='install_filename',
-        help="The file used to describe what should be installed\n  (defaults to $AUTOBUILD_CONFIG_FILE or \"autobuild.xml\").")
-    parser.add_argument(
-        '--installed-manifest',
-        default=configfile.INSTALLED_CONFIG_FILE,
-        dest='installed_filename',
-        help='The file used to record what is installed.')
-    parser.add_argument(
-        '-p', '--platform',
-        default=common.get_current_platform(),
-        dest='platform',
-        help='Override the automatically determined platform.')
-    parser.add_argument(
-        '--install-dir',
-        default=None,
-        dest='install_dir',
-        help='Where to unpack the installed files.')
-    parser.add_argument(
-        '--list',
-        action='store_true',
-        default=False,
-        dest='list_archives',
-        help="List the archives specified in the package file.")
-    parser.add_argument(
-        '--list-installed',
-        action='store_true',
-        default=False,
-        dest='list_installed',
-        help="List the installed package names and exit.")
-    parser.add_argument(
-        '--skip-license-check',
-        action='store_false',
-        default=True,
-        dest='check_license',
-        help="Do not perform the license check.")
-    parser.add_argument(
-        '--list-licenses',
-        action='store_true',
-        default=False,
-        dest='list_licenses',
-        help="List known licenses and exit.")
-    parser.add_argument(
-        '--export-manifest',
-        action='store_true',
-        default=False,
-        dest='export_manifest',
-        help="Print the install manifest to stdout and exit.")
-    parser.add_argument(
-        '--as-source',
-        action='append',
-        dest='as_source',
-        default=[],
-        help="Get the source for this package instead of prebuilt binary.")
-    parser.add_argument(
-        '--local',
-        action='append',
-        dest='local_archives',
-        default=[],
-        help="Install this locally built archive in place of the configured installable.")
 
 def print_list(label, array):
     """
@@ -537,21 +469,8 @@ def uninstall(package_name, installed_config):
                 # no idea what this exception is, better let it propagate
                 raise
 
-def install_packages(options, args):
-    # load the list of packages to install
-    logger.debug("loading " + options.install_filename)
-    config_file = configfile.ConfigurationDescription(options.install_filename)
-
-    # write packages into 'packages' subdir of build directory by default
-    if options.install_dir:
-        logger.debug("specified install directory: " + options.install_dir)
-    else:
-        build_directory = config_file.make_build_directory(options.dry_run)
-        options.install_dir = os.path.join(build_directory, 'packages')
-        logger.debug("default install directory: " + options.install_dir)
-
-    # get the absolute paths to the install dir and installed-packages.xml file
-    install_dir = os.path.realpath(options.install_dir)
+def install_packages(options, config_file, install_dir, args):
+    logger.debug("installing to directory: " + install_dir)
     # If installed_filename is already an absolute pathname, join() is smart
     # enough to leave it alone. Therefore we can do this unconditionally.
     installed_filename = os.path.join(install_dir, options.installed_filename)
@@ -570,14 +489,19 @@ def install_packages(options, args):
     # check the license properties for the packages to install
     if options.check_license:
         pre_install_license_check(packages, config_file)
-        
+
     # collect any locally built archives.
     local_archives = {}
     for archive_path in options.local_archives:
         try:
+            # split_tarname() returns a sequence like:
+            # ("/some/path", ["boost", "1.39.0", "darwin", "20100222a"], ".tar.bz2")
+            # We want just the package name, 'boost' in the example above.
             package = common.split_tarname(archive_path)[1][0]
-        except:
-            raise IntallError("cannot get package name from local archive " + archive_path)
+        except IndexError:
+            # But if the archive filename doesn't conform to our expectations,
+            # either subscript operation above might raise IndexError.
+            raise InstallError("cannot get package name from local archive " + archive_path)
         local_archives[package] = archive_path
 
     # do the actual install of the new/updated packages
@@ -606,10 +530,97 @@ class AutobuildTool(autobuild_base.AutobuildBase):
                     description='Fetch and install package archives.')
 
     def register(self, parser):
-        add_arguments(parser)
+        parser.description = "install artifacts of dependency packages for use during the build of the current package"
+        parser.add_argument(
+            'package',
+            nargs='*',
+            help='List of packages to consider for installation.')
+        parser.add_argument(
+            '--config-file',
+            default=configfile.AUTOBUILD_CONFIG_FILE,
+            dest='install_filename',
+            help="The file used to describe what should be installed\n  (defaults to $AUTOBUILD_CONFIG_FILE or \"autobuild.xml\").")
+        parser.add_argument(
+            '--installed-manifest',
+            default=configfile.INSTALLED_CONFIG_FILE,
+            dest='installed_filename',
+            help='The file used to record what is installed.')
+        parser.add_argument(
+            '-p', '--platform',
+            default=common.get_current_platform(),
+            dest='platform',
+            help='Override the automatically determined platform.')
+        parser.add_argument(
+            '--install-dir',
+            default=None,
+            dest='select_dir',          # see common.select_directories()
+            help='Where to unpack the installed files.')
+        parser.add_argument(
+            '--list',
+            action='store_true',
+            default=False,
+            dest='list_archives',
+            help="List the archives specified in the package file.")
+        parser.add_argument(
+            '--list-installed',
+            action='store_true',
+            default=False,
+            dest='list_installed',
+            help="List the installed package names and exit.")
+        parser.add_argument(
+            '--skip-license-check',
+            action='store_false',
+            default=True,
+            dest='check_license',
+            help="Do not perform the license check.")
+        parser.add_argument(
+            '--list-licenses',
+            action='store_true',
+            default=False,
+            dest='list_licenses',
+            help="List known licenses and exit.")
+        parser.add_argument(
+            '--export-manifest',
+            action='store_true',
+            default=False,
+            dest='export_manifest',
+            help="Print the install manifest to stdout and exit.")
+        parser.add_argument(
+            '--as-source',
+            action='append',
+            dest='as_source',
+            default=[],
+            help="Get the source for this package instead of prebuilt binary.")
+        parser.add_argument(
+            '--local',
+            action='append',
+            dest='local_archives',
+            default=[],
+            help="Install this locally built archive in place of the configured installable.")
+        parser.add_argument('--all','-a',dest='all', default=False, action="store_true",
+            help="install packages for all configurations")
+        parser.add_argument('--configuration', '-c', nargs='?', action="append", dest='configurations',
+                            help="install packages for a specific build configuration\n(may be specified as comma separated values in $AUTOBUILD_CONFIGURATION)",
+                            metavar='CONFIGURATION',
+                            default=self.configurations_from_environment())
 
     def run(self, args):
-        install_packages(args, args.package)
+        # load the list of packages to install
+        logger.debug("loading " + args.install_filename)
+        config = configfile.ConfigurationDescription(args.install_filename)
+
+        # write packages into 'packages' subdir of build directory by default
+        install_dirs = \
+             common.select_directories(args, config,
+                                       "install", "installing packages for",
+                                       lambda cnf:
+                                       os.path.join(config.make_build_directory(cnf, args.dry_run),
+                                                    "packages"))
+
+        # get the absolute paths to the install dir and installed-packages.xml file
+        for install_dir in install_dirs:
+            install_dir = os.path.realpath(install_dir)
+            install_packages(args, config, install_dir, args.package)
 
 if __name__ == '__main__':
     sys.exit("Please invoke this script using 'autobuild %s'" %
