@@ -94,17 +94,37 @@ class AutobuildTool(autobuild_base.AutobuildBase):
             default=None,
             dest='archive_format',
             help='the format of the archive (tbz2 or zip)')
+        parser.add_argument(
+            '--build-dir',
+            default=None,
+            dest='select_dir',          # see common.select_directories()
+            help='Package specific build directory.')
+        parser.add_argument('--all','-a',dest='all', default=False, action="store_true",
+            help="package all configurations")
+        parser.add_argument('--configuration', '-c', nargs='?', action="append", dest='configurations', 
+                            help="package a specific build configuration\n(may be specified as comma separated values in $AUTOBUILD_CONFIGURATION)",
+                            metavar='CONFIGURATION',
+                            default=self.configurations_from_environment())
 
     def run(self, args):
+        logger.debug("loading " + args.autobuild_filename)
         config = configfile.ConfigurationDescription(args.autobuild_filename)
-        package(config, args.platform, args.archive_filename, args.archive_format, args.check_license, args.dry_run)
+
+        build_dirs = common.select_directories(args, config, "build", "packaging",
+                                               lambda cnf:
+                                               config.get_build_directory(cnf, args.platform))
+
+        if not build_dirs:
+            build_dirs = [config.get_build_directory(None, args.platform)]
+        for build_dir in build_dirs:
+            package(config, build_dir, args.platform, args.archive_filename, args.archive_format, args.check_license, args.dry_run)
 
 
 class PackageError(AutobuildError):
     pass
 
 
-def package(config, platform_name, archive_filename=None, archive_format=None, check_license=True, dry_run=False):
+def package(config, build_directory, platform_name, archive_filename=None, archive_format=None, check_license=True, dry_run=False):
     """
     Create an archive for the given platform.
     """
@@ -118,7 +138,6 @@ def package(config, platform_name, archive_filename=None, archive_format=None, c
     print "packing %s" % package_description.name
     if not package_description.version:
         raise PackageError("no version number specified")
-    build_directory = config.get_build_directory(platform_name)
     logger.info('packaging from %r' % build_directory)
     if not os.path.isdir(build_directory):
         raise PackageError("build directory %s is not a directory" % build_directory)
@@ -141,6 +160,7 @@ def package(config, platform_name, archive_filename=None, archive_format=None, c
         tarfilename = archive_filename
     else:
         tarfilename = os.path.abs(os.path.join(config_directory, archive_filename))
+    logger.debug(tarfilename)
     if dry_run:
         for f in files:
             logger.info('added ' + f)
@@ -218,8 +238,9 @@ def _create_tarfile(tarfilename, build_directory, filelist):
             try:
                 tfile.add(file)
                 logger.info('added ' + file)
-            except:
-                raise PackageError("unable to add %s to %s" % (file, tarfilename))
+            except (tarfile.TarError, IOError), err:
+                # IOError in case the specified filename can't be opened
+                raise PackageError("unable to add %s to %s: %s" % (file, tarfilename, err))
         tfile.close()
     finally:
         os.chdir(current_directory)
@@ -227,8 +248,8 @@ def _create_tarfile(tarfilename, build_directory, filelist):
     # Downstream build tools utilize this output
     print "wrote  %s" % tarfilename
     _print_hash(tarfilename)
-    
-    
+
+
 def _create_zip_archive(archive_filename, build_directory, file_list):
     if not os.path.exists(os.path.dirname(archive_filename)):
         os.makedirs(os.path.dirname(archive_filename))
@@ -265,7 +286,7 @@ def _add_file_to_zip_archive(zip_file, unnormalized_file, archive_filename, adde
             raise PackageError("%s: unable to add %s to %s: %s" %
                                (err.__class__.__name__, file, archive_filename, err))
         logger.info('added ' + file)
-    
+
 
 def _print_hash(filename):
     fp = open(filename, 'rb')
