@@ -35,13 +35,6 @@ import re
 
 import common
 
-# Like any() but calls each element of the iterable
-def _any_call(iterable):
-    for element in iterable:
-        if element():
-            return True
-    return False
-
 class ExecutableError(common.AutobuildError):
     pass
 
@@ -79,15 +72,17 @@ class Executable(common.Serialized):
     def __call__(self, options=[], environment=os.environ):
         filters = self.get_filters()
         if filters:
+            print "Filtering: " + ", ".join(repr(f) for f in filters)
+            sys.stdout.flush()
             filters_re = [ re.compile(filter, re.MULTILINE) for filter in filters ]
             process = subprocess.Popen(' '.join(self._get_all_arguments(options)), shell=True, env=environment, stdout=subprocess.PIPE)
             for line in process.stdout:
-                filters_l = [ lambda: filter_re.search(line) for filter_re in filters_re ]
-                if _any_call(filters_l):
+                if any(regex.search(line) for regex in filters_re):
                     continue
                 line = line.replace("\r\n", "\n")
                 line = line.replace("\r", "\n")
-                print line, # Trailing , prevents an extra newline
+                sys.stdout.write(line)
+                sys.stdout.flush()
             return process.wait()
         else:
             return subprocess.call(' '.join(self._get_all_arguments(options)), shell=True, env=environment)
